<?php
/**
 *
 */
class plgHikashoppaymentAdyen extends hikashopPaymentPlugin
{
	var $accepted_currencies = array('EUR', 'USD');
	var $multiple = true;
	var $name = 'adyen';

	/**
	 * backend spec config
	 */
	var $pluginConfig = array(
		'merchantaccount' => array('Merchant Account', 'input'),
		'hmacKey' => array('Hmac Key', 'input'),
		'skincode' => array('Skin Code', 'input'),
		'url' => array('Adyen Payment Platform URL', 'input'),
		'language_code' => array('shopper Locale', 'input'),
		'notification' => array('allow notification from adyen', 'boolean', '0'),
		'debug' => array('DEBUG', 'boolean', '0'),
		'invalid_status' => array('INVALID_STATUS', 'orderstatus'),
		'pending_status' => array('PENDING_STATUS', 'orderstatus'),
		'verified_status' => array('VERIFIED_STATUS', 'orderstatus'),
		'allowedmethods' => array('Allowed Methods',' input'),
		'blockedmethods' => array('Blocked Methods', 'input'),
		'sessionvalidity' => array('Session Validity (Days)', 'input')

	);

	/**
	 *
	 */
	function onAfterOrderConfirm(&$order, &$methods, $method_id)
	{
		parent::onAfterOrderConfirm($order, $methods, $method_id);

		if (empty($this->payment_params->sessionvalidity)) {
			$this->payment_params->sessionvalidity = 1;
		}

		if (empty($this->payment_params->merchantaccount))
		{
			$this->app->enqueueMessage('You have to configure a merchant account for the adyen plugin payment first : check your plugin\'s parameters, on your website backend','error');
			return false;
		}

		if (empty($this->payment_params->skincode))
		{
			$this->app->enqueueMessage('You have to configure the Skin Code for the adyen plugin payment first : check your plugin\'s parameters, on your website backend','error');
			return false;
		}
		elseif (empty($this->payment_params->hmacKey))
		{
			$this->app->enqueueMessage('You have to configure the Hmac Key for the adyen plugin payment first : check your plugin\'s parameters, on your website backend','error');
			return false;
		}

		$amount = round($order->cart->full_total->prices[0]->price_value_with_tax, 2) * 100;

		//Setting address Data
		list($shipping_house, $shipping_street) = $this->splitStreet($order->cart->shipping_address->address_street);
		list($billing_house, $billing_street) = $this->splitStreet($order->cart->billing_address->address_street);

		//required data for adyen platform
		$resURL = HIKASHOP_LIVE . 'index.php?option=com_hikashop&ctrl=checkout&task=notify&notif_payment=adyen&tmpl=component' . $this->url_itemid;

		$vars = array(
			'allowedMethods' => preg_replace('#[^a-z0-9_\-,]#i', '', $this->payment_params->allowedmethods),
			'blockedMethods' => preg_replace('#[^a-z0-9_\-,]#i', '', $this->payment_params->blockedmethods),
			'currencyCode' => $this->currency->currency_code,
			'merchantAccount' => trim($this->payment_params->merchantaccount),
			'merchantReference' => $order->order_id,
			'paymentAmount' => $amount,
			'resURL' => $resURL,
			'sessionValidity' => date('c',strtotime('+' . (int)$this->payment_params->sessionvalidity . ' days')),
			'shipBeforeDate' => date('c', strtotime('+3 days')),
			'shopperEmail' => $this->user->user_email,
			'shopperLocale' => trim($this->payment_params->language_code),
			'shopperReference' => $this->user->user_id,
			'skinCode' => trim($this->payment_params->skincode),
			'billingAddress.city' => $order->cart->billing_address->address_city,
			'billingAddress.country' => $order->cart->billing_address->address_country->zone_code_2,
			'billingAddress.houseNumberOrName' => $billing_house,
			'billingAddress.postalCode' => @$order->cart->billing_address->address_post_code,
			'billingAddress.stateOrProvince' => $order->cart->billing_address->address_state->zone_name,
			'billingAddress.street' => $billing_street,
			'shopper.firstName' => @$order->cart->billing_address->address_firstname,
			'shopper.lastName' => @$order->cart->billing_address->address_lastname,
			'shopper.telephoneNumber' => @$order->cart->billing_address->address_telephone,
		);

		//create hash->Hmac/adyen
		$vars['merchantSig'] = $this->calculateSig($vars);

		$this->vars = $vars;
		return $this->showPage('end');
	}

	function calculateSig($vars) {
		ksort($vars, SORT_STRING);
		$data = array();
		foreach($vars as $key => $value) {
			$data[$key] = str_replace(':','\\:', str_replace('\\', '\\\\', $value));
		}
		$signingString = implode(":", array_merge(array_keys($data), array_values($data)));
		$binaryHmacKey = pack("H*" , trim($this->payment_params->hmacKey));
		$binaryHmac = hash_hmac('sha256', $signingString, $binaryHmacKey, true);
		$signature = base64_encode($binaryHmac);
		return $signature;
	}

	/**
	 * Split Street Number from billing adress
	 */
	function splitStreet($street)
	{
		if (preg_match('#[0-9]+#',$street,$matches))
		{
			return array($matches[0], preg_replace('#'.$matches[0].'#', '', $street, 1));
		}
		return array('', $street);
	}

	/**
	 * set the specific configuration (back end) default values
	 */
	function getPaymentDefaultValues(&$element)
	{
		$element->payment_name = 'adyen';
		$element->payment_description = 'You can pay by credit card using this payment method';
		$element->payment_images = 'MasterCard,VISA,Credit_card,American_Express';
		$element->payment_params->notification = 1;
		$element->payment_params->url = 'https://test.adyen.com/hpp/pay.shtml';
		$element->payment_params->invalid_status = 'cancelled';
		$element->payment_params->verified_status = 'confirmed';
		$element->payment_params->pending_status = 'created';
	}

	function calculateNotifSig($vars) {
		$data = array();
		foreach($vars as $key => $value){
			if($key != 'merchantSig')
				$data[$key] = $value;
		}
		$signingString = implode(":", array_merge(array_keys($data), array_values($data)));
		$binaryHmacKey = pack("H*" , trim($this->payment_params->hmacKey));
		$binaryHmac = hash_hmac('sha256', $signingString, $binaryHmacKey, true);
		$signature = base64_encode($binaryHmac);
		return $signature;
	}

	/**
	 * treatment on response information from Ayden servers
	 */
	function onPaymentNotification(&$statuses)
	{
		//filtered array from the parameters received
		$vars = array();
		$filter = JFilterInput::getInstance();
		foreach($_REQUEST as $key => $value)
		{
			$key = $filter->clean($key);
			$value = hikaInput::get()->getString($key);
			$vars[$key]=$value;
		}

		//TWe load the parameters of the plugin
		$order_id = (int)@$vars['merchantReference'];
		$dbOrder = $this->getOrder($order_id);
		$this->loadPaymentParams($dbOrder);
		if(empty($this->payment_params))
			return false;
		$this->loadOrderData($dbOrder);

		//"succes URL" and the "fail URL"
		$return_url = HIKASHOP_LIVE.'index.php?option=com_hikashop&ctrl=checkout&task=after_end&order_id='.$order_id.$this->url_itemid;
		$cancel_url = HIKASHOP_LIVE.'index.php?option=com_hikashop&ctrl=order&task=cancel_order&order_id='.$order_id.$this->url_itemid;

		if(!$this->payment_params->notification)
		{
			$this->app->redirect($return_url);
			return false;
		}

		if($this->payment_params->debug)
		{
			//debug information stored in the payment log file
			hikashop_writeToLog($vars);
			hikashop_writeToLog($dbOrder);
		}

		$hmacKey = $this->calculateNotifSig($vars);
		if($hmacKey != $vars['merchantSig']) {
			//debug
			if ($this->payment_params->debug)
				hikashop_writeToLog('Hash error ' . $vars['merchantSig'] . ' - ' . $hmacKey );
			return false;
		}

		if(!isset($vars['authResult']))
		{
			$vars['authResult'] = 'UNKNOWN';
			if($vars['eventCode'] === 'AUTHORISATION')
			{
				if($vars['success'] === 'true')
				{
					$vars['authResult'] = 'AUTHORISED';
				}
				else if(!empty($vars['reason']))
				{
					$vars['authResult'] = 'REFUSED';
				}
			}
		}

		//set the "invalid" status
		if ($vars['authResult'] != 'PENDING' && $vars['authResult'] != 'AUTHORISED')
		{
			//attribute invalid_status.
			$this->modifyOrder($order_id, $this->payment_params->invalid_status, true, true);

			$this->app->redirect($cancel_url);
			return false;
		}

		// Else: validated -> success
		if ($vars['authResult'] == 'AUTHORISED')
		{
			$order_status = $this->payment_params->verified_status;
		}
		else if ($vars['authResult'] == 'PENDING')
		{
			$order_status = $this->payment_params->pending_status;
		}

		if ($dbOrder->order_status != $order_status)
		{
			$this->modifyOrder($order_id, $order_status, true, true);
		}

		$this->app->redirect($return_url);
		return true;
	}
}